#include <avr/io.h>
#include <util/delay.h>
#include <stdlib.h>
#include "LC7980.h"

uint8_t lcd_cursor = 0;

void LCD_Strobe() {
	
	LCD_CTRL_PORT |= LCD_E;
	//_delay_us(1);
	LCD_CTRL_PORT &= ~LCD_E;
	_delay_us(5);
}

void LCD_WriteCommand(uint8_t value) {
	LCD_CTRL_PORT &= ~LCD_RW;
	LCD_CTRL_PORT |= LCD_RS;
	LCD_DATA_PORT = value;

	LCD_Strobe();

}

void LCD_WriteData(uint8_t value) {
	LCD_CTRL_PORT &= ~LCD_RW;
	LCD_CTRL_PORT &= ~(LCD_RS);
	
	LCD_DATA_PORT = value;

	LCD_Strobe();
}



void LCD_SetCursor() {
    LCD_WriteCommand(10);
    LCD_WriteData(0);

    LCD_WriteCommand(11);
    LCD_WriteData(0);
}

void LCD_Init() {
	LCD_DATA_DDR = 0xFF;
	LCD_CTRL_DDR |= LCD_E | LCD_RS | LCD_RW;
	LCD_WriteCommand(0);
    LCD_WriteData(LCD_DISPLAY_ON | LCD_BLINK_32 | LCD_TRANSFER_BIT | LCD_BLINK_ON | LCD_CURSOR_ON | LCD_MODE_TEXT | LCD_CG_INT);

	LCD_SetCharSize(6,8);
	LCD_SetDisplaySize(240/6,40);

	LCD_SetDisplayStart(0);

	LCD_SetCursor();

	LCD_WriteCommand(12);
	for(uint8_t i = 0; i < 255; i++) {
		LCD_WriteData(' ');

	}
	LCD_SetCursorAddr(0);
	LCD_WriteCommand(12);

}


//ok
//Sets the height and width of the characters on the display
//in graphic mode only width is mandatory
void LCD_SetCharSize(uint8_t width, uint8_t height) {
	LCD_WriteCommand(LCD_SET_CHAR_PITCH);
	LCD_WriteData(((height - 1)&0x0F)<<4 | ((width-1) & 0x07));
}

//Sets the width of the display.
//in text-mode the amount of characters ([Display width]/[width from SetCharSize])
//in graphic mode the bytes displayed per row (also dependend from SetCharSize?!)
void LCD_SetDisplaySize(uint8_t width, uint8_t height) {
	LCD_WriteCommand(LCD_SET_CHAR_NUM);
	LCD_WriteData(width - 1);

	LCD_WriteCommand(LCD_SET_TIME_DIV);
	LCD_WriteData(height - 1);
}

void LCD_SetDisplayStart(uint16_t addr) {
	LCD_WriteCommand(LCD_SET_START_LO);
	LCD_WriteData(addr & 0xFF);

	LCD_WriteCommand(LCD_SET_START_HI);
	LCD_WriteData(addr >> 8);
}

void LCD_SetCursorAddr(uint16_t pos) {
	LCD_WriteCommand(LCD_SET_CURS_LO);
	LCD_WriteData(pos & 0xFF);


	LCD_WriteCommand(LCD_SET_CURS_HI);
	LCD_WriteData(pos >> 8);	
}

void LCD_WriteDisplayData() {
	LCD_WriteCommand(LCD_WRITE_DISPLAY_DATA);
}

void LCD_ClearHome() {
	lcd_cursor = 0;
	LCD_SetCursorAddr(0);
	LCD_WriteCommand(12);
	for(uint8_t i = 0; i < 250; i++) {
		LCD_WriteData(' ');
	}
	LCD_SetCursorAddr(0);
	LCD_WriteCommand(12);
}

void LCD_PutC(char c) {
	if(lcd_cursor >= 200) {
		LCD_ClearHome();
	}
	LCD_WriteData(c);
	lcd_cursor++;
}

void LCD_PutS(char *s) {
	while(*s) {
		LCD_WriteData(*s++);
	}
}


void LCD_PutDHex(uint16_t hex) {
	uint16_t tmp;
	LCD_PutS("0x");
	for(uint8_t i = 0; i < 4; i++) {
		tmp = hex & 0xF000; //foo ;)
		tmp >>= 12;
		tmp += (tmp < 10) ? 48 : 55;
		hex <<= 4;
		LCD_PutC(tmp);
	}
}

void LCD_PutHex(uint8_t hex) {
	uint8_t tmp;
	LCD_PutS("0x");
	for(uint8_t i = 0; i < 2; i++) {
		tmp = hex & 0xF0;
		tmp >>= 4;
		tmp += (tmp < 10) ? 48 : 55;
		hex <<= 4;
		LCD_PutC(tmp);
	}
}


void LCD_PutNum(uint16_t num) {
	char buff[6];
	itoa(num, buff, 10);
	LCD_PutS(buff);
}
